<?php
/******************************************************************************
* Media.php                                                                   *
* =========================================================================== *
* Listen to music that users have uploaded to your forum                      *
*******************************************************************************
* Software version:               1.0                                         *
* Software by:                    Juan "JayBachatero" Hernandez               *
* Copyright (c) 2006 by:          Juan "JayBachatero" Hernandez               *
* Contact:                        Jay@JayBachatero.com                        *
* Website:                        JayBachatero.com                            *
* =========================================================================== *
* This mod is free software; you may redistribute it and/or modify it as long *
* as you credit me for the original mod. This mod is distributed in the hope  *
* that it is and will be useful, but WITHOUT ANY WARRANTIES; without even any *
* implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    *
*                                                                             *
* All SMF copyrights are still in effect. Anything not mine is theirs. Enjoy! *
******************************************************************************/
if (!defined('SMF'))
	die('Hacking attempt...');

/*	This file handles the listening, viewing and downloading the files.

	void MediaCenter()
		- This is the core function.  It handles all three types of files.
		- Checks to see if you are allowed to be in here at all.
		- Checks permission for each type of file.  Three different permissions.
		  Each one is combined with the media_center_enter permission.
		- Complext subActions array to handle filetypes, subaction and
		  sub_template that are associated with a sub action.
		  - array music
		    - This is the default action.  If no other action is selected this will be selected.
		    - The default  associated fileTypes are mp3, wav and wma.
		  - array videos
		    - Sets up the sub_template name and the fileTypes associated with it.
		    - Default associated fileTypes are wmv, mov, mpg, mpeg and avi.
		  - array images
		    - Like the other arrays it sets up the sub_template and fileTypes.
		    - Default associated fileTypes are png, jpeg, jpg, and gif.
		- This file also handles the sorting.  Default is set to filename
		- Two link trees are used.  One for the main media center page and
		  the other for the current sub_action
		- The attachment count function checks to see the attachments what the user is
		  allowed to see and filters the results by the current sub_action.
		  This query is also cached for each user.
		- This is a BIG query.  It actaully loads 6 tables, 7 if it's an image.
		  This query gets the basic information for the files.  It also handles viewing only files
		  that you are supposed to see.  This is to ensure that the user is not viewing a file that
		  they are not supposed to see.
		- Then after the query is done the data gets passed to the $attach array.  There is $attach['file']
		  and $attach['download'].  These two handle the popups for the actual files to listen and/or view images & videos.
		  $attach['download'] is to download the file.
		- Sets up the $context for media_center, page_title and sub_template.

	void requestFile()
		- This function is a modified version of the Download() function in Display.php.
		- First checks the permission to see if you can even access the media center.
		- Fetches the attachment info
		- Increases the download cound for the attachments.
		- Sends some headers for downloading and streaming.
		- Sends the Content-Length header so that Firefox wont chop the song.

	void MediaPopup()
		- Sets up the context for the popus used for music and videos.
		- Removes all the loaded templates for a clean popup.
*/

function Media()
{
	global $context, $scripturl, $modSettings, $ID_MEMBER, $db_prefix, $txt, $user_info, $func, $settings;

	// Load the language and the template
	loadLanguage('MediaCenter');
	loadTemplate('MediaCenter');

	// Set the permission to listen or view images/video
	$context['can_listen'] = allowedTo('media_center_listen');
	$context['can_see_videos'] = allowedTo('media_center_videos');
	$context['can_see_images'] = allowedTo('media_center_images');
	$context['can_see_custom'] = allowedTo('media_center_custom');

	// Determine the default action.
	if ($context['can_listen'])
		$default = 'music';
	elseif ($context['can_see_videos'])
		$default = 'videos';
	elseif ($context['can_see_images'])
		$default = 'images';

	// Can they be in here
	if (!$context['can_listen'] && !$context['can_see_videos'] && !$context['can_see_images'] && !$context['can_see_custom'])
		isAllowedTo('media_center_custom');

	/*
		This section of the mod is very powerful.  With just a single addition to this array
		you can add a whole new section of files to support.  If you wanted to add an archive
		section all you would have to do is add to the array.
			'archives' => array(
				'title' => $txt['media_tabs_customname'],
				'sub_template' => 'media',
				'fileTypes' => array('zip', 'tar'),
				'href' => $scripturl . '?action=mediacenter;sa=archives',
				'is_allowed' => $context['can_see_custom'],
				'is_last' => true,
			),
		The first thing is the title of the custom area.  You need to add a language entry to
		MediaCenter.english.php.  It must follow this format. $txt['media_tabs_customname'].
		Replace customname with the key used for the array.  The sub_template should be set to media.
		If changed to images it will not work correctly.  The next one is fileTypes.  This is where
		you define the file extensions that are associated with the section.  So if you wanted to add
		zip and tar files then you would use array('zip', 'tar').  The href key is where you define the
		link.  Use the same name that you used for the key in the array.  In this case "archives".
		The is_allowed to checks the permission to see if they can see this section.  The use of
		$context['can_see_custom'] is recommended since it's used in other places through out the mod.
		If this is the last section of the mod then you need to add 'is_last' => true to the array.
	*/
	$subActions = array(
		'music' => array(
			'title' => $txt['media_tabs_music'],
			'sub_template' => 'media',
			'fileTypes' => array('mp3', 'wav', 'wma'),
			'href' => $scripturl . '?action=mediacenter;sa=music',
			'is_allowed' => $context['can_listen'],
		),
		'videos' => array(
			'title' => $txt['media_tabs_videos'],
			'sub_template' => 'media',
			'fileTypes' => array('wmv', 'mov', 'mpg', 'mpeg', 'avi'),
			'href' => $scripturl . '?action=mediacenter;sa=videos',
			'is_allowed' => $context['can_see_videos'],
		),
		'images' => array(
			'title' => $txt['media_tabs_images'],
			'sub_template' => 'images',
			'fileTypes' => array('png', 'jpeg', 'jpg', 'gif'),
			'href' => $scripturl . '?action=mediacenter;sa=images',
			'is_allowed' => $context['can_see_images'],
			'is_last' => true,
		),
	);

	// Set the subaction
	$_REQUEST['sa'] = isset($_REQUEST['sa']) && isset($subActions[$_REQUEST['sa']]) ? $_REQUEST['sa'] : $default;
	$context['sub_action'] = $_REQUEST['sa'];

	// Check what tab is selected
	if (isset($subActions[$context['sub_action']]))
		$subActions[$context['sub_action']]['is_selected'] = true;

	// Set the sort methods
	$sort_methods = array(
		'filename' => 'att.filename',
		'board' => 'b.board',
		'filesize' => 'att.size',
		'downloads' => 'att.downloads',
		'poster' => 'posterName',
		'subject' => 'm.subject',
	);

	// Sort by filename asc.
	if (!isset($_REQUEST['sort']) || !isset($sort_methods[$_REQUEST['sort']]))
	{
		$context['sort_by'] = 'filename';
		$_REQUEST['sort'] = 'att.filename';
		$ascending = true;
	}
	// Otherwise default to desc.
	else
	{
		$context['sort_by'] = $_REQUEST['sort'];
		$_REQUEST['sort'] = $sort_methods[$_REQUEST['sort']];
		$ascending = !isset($_REQUEST['desc']);
	}

	$context['sort_direction'] = $ascending ? 'up' : 'down';

	// Set the link tree
	$context['linktree'][] = array(
		'url' => $scripturl . '?action=mediacenter',
		'name' => $txt['media_title'],
	);

	// The extra tree for the sa
	$context['linktree'][] = array(
		'url' => $scripturl . '?action=mediacenter;sa=' . $context['sub_action'],
		'name' => $txt['media_tabs_' . $context['sub_action']],
	);

	// Initialize start
	$context['start'] = (!isset($_REQUEST['start']) || $_REQUEST['start'] < 0) ? 0 : (int) $_REQUEST['start'];

	// Get the fileTypes that we are working with.
	$fileTypes = $subActions[$context['sub_action']]['fileTypes'];

	// Check to see if the extension is 4 chars long
	$fileTypesLong = array();
	foreach ($fileTypes as $type)
		if (strlen($type) == 4)
		{
			$fileTypesLong[] = $type;
			unset($fileTypes[$type]);
		}

	// Make the clause for the query
	$fileTypes = "(RIGHT(att.filename, 3) = '" . implode("' OR RIGHT(att.filename, 3) = '", $fileTypes) . "'" . (empty($fileTypesLong) ? '' : " OR RIGHT(att.filename, 4) = '" . implode("' OR RIGHT(att.filename, 4) = '", $fileTypesLong) . "'") . ")";

	// Do we want results just from a board?
	if (isset($_REQUEST['mediaboards']) && (!is_array($_REQUEST['mediaboards']) && !empty($_REQUEST['mediaboards'])) && !isset($topicUrl))
	{
		// Explode it.
		$boardsArray = explode(',', $_REQUEST['mediaboards']);

		// Make sure it's an integer
		$boards = '';
		foreach ($boardsArray as $board)
			if (!empty($board))
				$boards .= (int) $board . ',';

		$boardQuery = "b.ID_BOARD IN (" . rtrim($boards, ',') . ")";

		// Url
		$boardUrl = rtrim($boards, ',');
		$context['board_url'] = $boardUrl;
	}
	else
		$boardQuery = '1';

	// Are we looking for files in a specific topic?
	if (isset($_REQUEST['mediatopics']) && (!is_array($_REQUEST['mediatopics']) && !empty($_REQUEST['mediatopics'])) && !isset($boardUrl))
	{
		// Explode it.
		$topicsArray = explode(',', $_REQUEST['mediatopics']);

		// Make sure it's an integer
		$topics = '';
		foreach ($topicsArray as $topic)
			if (!empty($topic))
				$topics .= (int) $topic . ',';

		$topicQuery = "m.ID_TOPIC IN (" . rtrim($topics, ',') . ")";

		// Url
		$topicUrl = rtrim($topics, ',');
		$context['topic_url'] = $topicUrl;
	}
	else
		$topicQuery = '1';

	/*
		Set the ttl time for each cache level
		If at level 1 then just cache for 2 mins, level 2 for 4 mins and level 3 for 6 mins.
	*/
	if (!empty($modSettings['cache_enable']) && $modSettings['cache_enable'] == 1)
		$cache_time = 120;
	elseif (!empty($modSettings['cache_enable']) && $modSettings['cache_enable'] == 2)
		$cache_time = 240;
	elseif (!empty($modSettings['cache_enable']) && $modSettings['cache_enable'] == 3)
		$cache_time = 480;
	else
		$cache_time = 0;

	// Set this var to use if topicUrl or boardUrl is set
	if (!empty($topicUrl))
		$url = 'topic' . $topicUrl;
	elseif (!empty($boardUrl))
		$url = 'board' . $boardUrl;
	else
		$url = $context['sub_action'];

	if (!isset($_SESSION['attCount'][$url][implode(',', $user_info['groups'])]) || (isset($_SESSION['attCount'][$url][implode(',', $user_info['groups'])]) && time() - $_SESSION['attCount'][$url][implode(',', $user_info['groups'])]['time'] > $cache_time))
	{
		// Select query to get the amount of mods
		$request = db_query("
			SELECT COUNT(*)
			FROM ({$db_prefix}attachments AS att, {$db_prefix}boards AS b, {$db_prefix}messages AS m)
			WHERE m.ID_BOARD = b.ID_BOARD
				AND att.ID_MSG = m.ID_MSG
				AND $user_info[query_see_board]
				AND $fileTypes
				AND $topicQuery
				AND $boardQuery", __FILE__, __LINE__);
		list ($attCount) = mysql_fetch_row($request);
		mysql_free_result($request);

		$_SESSION['attCount'][$url][implode(',', $user_info['groups'])]['count'] = $attCount;
		$_SESSION['attCount'][$url][implode(',', $user_info['groups'])]['time'] = time();
	}
	else
		$attCount = (int) $_SESSION['attCount'][$url][implode(',', $user_info['groups'])]['count'];

	// If $attCount is empty then we must set it to 1
	$attCount = empty($attCount) ? 1 : $attCount;

	// This method was taken from MessageIndex.php to make sorting faster.
	$start = $context['start'];
	$maxindex = isset($_REQUEST['limit']) ? (int) $_REQUEST['limit'] : $modSettings['defaultMaxTopics'];
	if ($start > ($attCount - 1) / 2)
	{
		$ascending = !$ascending;
		$fake_ascending = true;
		$maxindex = $attCount < $start + $maxindex + 1 ? $attCount - $start : $maxindex;
		$start = $attCount < $start + $maxindex + 1 ? 0 : $attCount - $start - $maxindex;
	}
	else
		$fake_ascending = false;

	// Is $start a negative number?
	$maxindex = $maxindex < 0 ? $modSettings['defaultMaxTopics'] : $maxindex;

	// Contrusct the page index
	if (isset($_REQUEST['sort']))
		$context['page_index'] = constructPageIndex($scripturl . '?action=mediacenter;sa=' . $context['sub_action'] .';sort=' . $context['sort_by'] . (!empty($_REQUEST['desc']) ? ';desc' : '') . (isset($topicUrl) ? ';mediatopics=' . $topicUrl: '') . (isset($boardUrl) ? ';mediaboards=' . $boardUrl : ''), $context['start'], $attCount, $modSettings['defaultMaxTopics']);
	else
		$context['page_index'] = constructPageindex($scripturl . '?action=mediacenter;sa=' . $context['sub_action'] . (isset($topicUrl) ? ';mediatopics=' . $topicUrl: '') . (isset($boardUrl) ? ';mediaboards=' . $boardUrl: ''), $context['start'], $attCount, $modSettings['defaultMaxTopics']);

	/*
		The name for the cache is kind long.  It's divided by a _
		First section is the name of the script "media-center"
		Second the current sub action or topic if we are viewing the items in a topic.
		Third is the ID_GROUP in a CSV list
		Fourth is the current possition for $start
	*/
	$cache_name = 'media-center_' . ($topicQuery == '1' ? $_REQUEST['sa'] : 'topic' . $topicUrl) . '_' . implode(',', $user_info['groups']) . '_' . $context['start'];

	// Select the results
	if (($ID_ATTACH = cache_get_data($cache_name, $cache_time)) == null)
	{
		$request = db_query("
			SELECT att.ID_ATTACH
			FROM ({$db_prefix}attachments AS att, {$db_prefix}messages AS m, {$db_prefix}boards AS b)
			WHERE att.ID_MSG = m.ID_MSG
				AND b.ID_BOARD = m.ID_BOARD
				AND $fileTypes
				AND $user_info[query_see_board]
				AND $topicQuery
				AND $boardQuery
			ORDER BY $_REQUEST[sort] " . ($ascending ? '' : ' DESC'). "
			LIMIT $start, $maxindex", __FILE__, __LINE__);

		// Get the is of the attachments and cache them.
		$ID_ATTACH = array();
		while ($row = mysql_fetch_assoc($request))
			$ID_ATTACH[] = $row['ID_ATTACH'];

		if (!empty($ID_ATTACH))
			$ID_ATTACH = implode(',', $ID_ATTACH);
		else
			$ID_ATTACH = '0';

		// Cache the ID's if enabled
		if (!empty($modSettings['cache_enable']))
			cache_put_data($cache_name, $ID_ATTACH, $cache_time);
	}

	$request = db_query("
		SELECT
			att.ID_ATTACH, att.ID_MSG, att.filename, IFNULL(att.size, 0) AS filesize, att.downloads, mem.ID_MEMBER,
			IFNULL(mem.realName, m.posterName) AS posterName, m.ID_TOPIC, m.subject, t.ID_BOARD, m.posterTime" . ($context['sub_action'] == 'images' && !empty($modSettings['attachmentThumbnails'])? ",
			att.width, att.height, IFNULL(thumb.ID_ATTACH, 0) AS ID_THUMB, thumb.width AS thumb_width, thumb.height AS thumb_height" : '') . "
		FROM ({$db_prefix}attachments AS att, {$db_prefix}messages AS m, {$db_prefix}topics AS t)
			LEFT JOIN {$db_prefix}members AS mem ON (mem.ID_MEMBER = m.ID_MEMBER)" . ($context['sub_action'] == 'images' && !empty($modSettings['attachmentThumbnails']) ? "
			LEFT JOIN {$db_prefix}attachments AS thumb ON (thumb.ID_ATTACH = att.ID_THUMB)" : '') . "
		WHERE att.ID_ATTACH IN ($ID_ATTACH)
			AND t.ID_TOPIC = m.ID_TOPIC
			AND att.ID_MSG = m.ID_MSG
		ORDER BY $_REQUEST[sort] " . ($ascending ? '' : ' DESC'), __FILE__, __LINE__);

	$attach = array();
	while ($row = mysql_fetch_assoc($request))
	{
		$attach[$row['ID_ATTACH']] = array(
			'member' => array(
				'id' => $row['ID_MEMBER'],
				'name' => $row['posterName'],
				'link' => empty($row['ID_MEMBER']) ? $row['posterName'] : '<a href="' . $scripturl . '?action=profile;u=' . $row['ID_MEMBER'] . '">' . $row['posterName'] . '</a>',
			),
			'file' => array(
				'filename' => empty($modSettings['media_disable_filename_filter']) ? $func['ucwords'](strtolower(str_replace(array('_', '/', '\\', '-'), array(' ', '', '', ' - '), $row['filename']))) : $row['filename'],
				'filesize' => round($row['filesize'] / 1024, 2) . $txt['smf211'],
				'download' => $scripturl . '?action=mediacenterfile;attach=' . $row['ID_ATTACH'] . ';custom;download',
				'downloads' => $row['downloads'],
			),
			'topic' => array(
				'id' => $row['ID_TOPIC'],
				'ID_BOARD' => $row['ID_BOARD'],
				'subject' => str_replace($txt['response_prefix'], '', $row['subject']),
				'link' => '<a href="' . $scripturl . '?topic=' . $row['ID_TOPIC'] . '.msg' . $row['ID_MSG'] . '#msg' . $row['ID_MSG'] . '">' . $func['ucwords'](strtolower($row['subject'])) . '</a>',
				'time' => timeformat($row['posterTime']),
			),
		);

		// Get the audio file.
		if ($context['sub_action'] == 'music' && $context['can_listen'])
			$attach[$row['ID_ATTACH']]['file'] += array(
				'href' => $scripturl . '?action=mediacenterpopup;attach=' . $row['ID_ATTACH'] . ';music',
				'download' => $scripturl . '?action=mediacenterfile;attach=' . $row['ID_ATTACH'] . ';music;download',
			);

		// Are we looking at videos?
		if ($context['sub_action'] == 'videos' && $context['can_see_videos'])
			$attach[$row['ID_ATTACH']]['file'] += array(
				'href' => $scripturl . '?action=mediacenterpopup;attach=' . $row['ID_ATTACH'] . ';video',
				'download' => $scripturl . '?action=mediacenterfile;attach=' . $row['ID_ATTACH'] . ';video;download',
			);

		// Niceeeeee.  We looking at images now.  Here we need some more info ;).
		if ($context['sub_action'] == 'images' && $context['can_see_images'])
			$attach[$row['ID_ATTACH']]['file'] += array(
				'href' => '<a href="' . $scripturl . '?action=mediacenterfile;attach=' . $row['ID_ATTACH'] . ';image" onclick="return reqWin(\'' .$scripturl . '?action=mediacenterfile;attach=' . $row['ID_ATTACH'] . ';image\', \'' . ($row['width'] + 40) . '\', \'' . ($row['height'] + 40) . '\', true);"><img src="' . (($row['width'] <= $modSettings['attachmentThumbWidth'] && $row['height'] <= $modSettings['attachmentThumbHeight']) ? $scripturl . '?action=mediacenterfile;attach=' . $row['ID_ATTACH'] . ';image' : (empty($row['thumb_width']) ? $settings['default_theme_url'] . '/images/media_no_thumb.gif' : $scripturl . '?action=mediacenterfile;attach=' . $row['ID_THUMB'] . ';image')) . '" alt="' . $attach[$row['ID_ATTACH']]['file']['filename'] . '" /></a>',
				'download' => $scripturl . '?action=mediacenterfile;attach=' . $row['ID_ATTACH'] . ';image;download',
				'width' => $row['width'],
				'height' => $row['height'],
			);
	}

	// That monkey.  That funky monkey :P.
	if (isset($_REQUEST['monkey'])) $context['special'] = '<div align="center"><object width="425" height="350"><param name="movie" value="http://www.youtube.com/v/i3cv9_xqPwA"></param><param name="wmode" value="transparent"></param><embed src="http://www.youtube.com/v/i3cv9_xqPwA" type="application/x-shockwave-flash" wmode="transparent" width="425" height="350"></embed></object></div>'; else $context['special'] = '';

	if ($fake_ascending)
		$attach = array_reverse($attach, true);

	// Set the context
	$context['media_center'] = array(
		'attach' => $attach,
		'tabs' => $subActions,
	);

	// Load the subtemplate
	$context['sub_template'] = $subActions[$_REQUEST['sa']]['sub_template'];

	// Title
	$context['page_title'] = $txt['media_title'] . ' - ' . $txt['media_tabs_' . $context['sub_action']];

}

function RequestFile()
{
	global $db_prefix, $modSettings, $context, $txt;

	// Check if they are allowed to be here.
	if (isset($_REQUEST['music']) && !allowedTo('media_center_listen'))
		exit;
	elseif (isset($_REQUEST['video']) && !allowedTo('media_center_video'))
		exit;
	elseif (isset($_REQUEST['image']) && !allowedTo('media_center_listen'))
		exit;
	elseif (isset($_REQUEST['custom']) && !allowedTo('media_center_custom'))
		exit;
	elseif (!isset($_REQUEST['music']) && !isset($_REQUEST['video']) && !isset($_REQUEST['image']) && !isset($_REQUEST['custom']))
		exit;

	// If attach not set just exit.
	if (!isset($_REQUEST['attach']) || empty($_REQUEST['attach']))
		exit;
	else
		$_REQUEST['attach'] = (int) $_REQUEST['attach'];

	$context['no_last_modified'] = true;

	// Get the thumb
	$request = db_query("
		SELECT att.filename, att.ID_ATTACH
		FROM ({$db_prefix}boards AS b, {$db_prefix}messages AS m, {$db_prefix}attachments AS att)" . (isset($_REQUEST['image']) ? "
			LEFT JOIN {$db_prefix}attachments AS thumb ON (thumb.ID_ATTACH = att.ID_THUMB)" : '') . "
		WHERE b.ID_BOARD = m.ID_BOARD
			AND $user_info[query_see_board]
			AND m.ID_MSG = att.ID_MSG
			AND att.ID_ATTACH = $_REQUEST[attach]
		LIMIT 1", __FILE__, __LINE__);
	list ($real_filename, $ID_ATTACH) = mysql_fetch_row($request);
	mysql_free_result($request);

	// Just exit if the attachment wasnt found
	if (empty($real_filename))
		exit;

	// Increase the download count.
	if (isset($_REQUEST['download']))
		db_query("
			UPDATE {$db_prefix}attachments
			SET downloads = downloads + 1
			WHERE ID_ATTACH = $_REQUEST[attach]
			LIMIT 1", __FILE__, __LINE__);

	$filename = getAttachmentFilename($real_filename, $_REQUEST['attach']);

	// This is done to clear any output that was made before now. (would use ob_clean(), but that's PHP 4.2.0+...)
	ob_end_clean();
	if (!empty($modSettings['enableCompressedOutput']) && @version_compare(PHP_VERSION, '4.2.0') >= 0 && @filesize($filename) <= 4194304)
		@ob_start('ob_gzhandler');
	else
	{
		ob_start();
		header('Content-Encoding: none');
	}

	// No point in a nicer message, because this is supposed to be an attachment anyway...
	if (!file_exists($filename))
	{
		loadLanguage('Errors');

		header('HTTP/1.0 404 ' . $txt['attachment_not_found']);
		header('Content-Type: text/plain; charset=' . (empty($context['character_set']) ? 'ISO-8859-1' : $context['character_set']));

		// We need to die like this *before* we send any anti-caching headers as below.
		die('404 - ' . $txt['attachment_not_found']);
	}

	// If it hasn't been modified since the last time this attachement was retrieved, there's no need to display it again.
	if (!empty($_SERVER['HTTP_IF_MODIFIED_SINCE']))
	{
		list($modified_since) = explode(';', $_SERVER['HTTP_IF_MODIFIED_SINCE']);
		if (strtotime($modified_since) >= filemtime($filename))
		{
			ob_end_clean();

			// Answer the question - no, it hasn't been modified ;).
			header('HTTP/1.1 304 Not Modified');
			exit;
		}
	}

	// Check whether the ETag was sent back, and cache based on that...
	$file_md5 = '"' . md5_file($filename) . '"';
	if (!empty($_SERVER['HTTP_IF_NONE_MATCH']) && strpos($_SERVER['HTTP_IF_NONE_MATCH'], $file_md5) !== false)
	{
		ob_end_clean();

		header('HTTP/1.1 304 Not Modified');
		exit;
	}

	// Send the attachment headers.
	header('Pragma: ');
	header('Cache-Control: max-age=' . (525600 * 60) . ', private');
	if (!$context['browser']['is_gecko'])
		header('Content-Transfer-Encoding: binary');
	header('Expires: ' . gmdate('D, d M Y H:i:s', time() + 525600 * 60) . ' GMT');
	header('Last-Modified: ' . gmdate('D, d M Y H:i:s', filemtime($filename)) . ' GMT');
	header('Accept-Ranges: bytes');
	header('Set-Cookie:');
	header('Connection: close');
	header('ETag: ' . $file_md5);

	// Are they downloading it?
	if (isset($_REQUEST['download']))
		header('Content-Disposition: attachment; filename="' . $real_filename . '"');

	// Set the correct mime type.
	if (filesize($filename) != 0 && isset($_REQUEST['image']))
	{
		$size = @getimagesize($filename);
		if (!empty($size) && $size[2] > 0 && $size[2] < 4)
			header('Content-Type: image/' . ($size[2] != 1 ? ($size[2] != 2 ? 'png' : 'jpeg') : 'gif'));
		// Errr, it's an image.... what kind?  A... gif?  Yeah that's it, gif!  Like JIF, the peanut butter.
		elseif (isset($_REQUEST['image']))
			header('Content-Type: image/gif');
	}
	elseif (isset($_REQUEST['music']) || isset($_REQUEST['video']) || isset($_REQUEST['custom']))
	{
		// Get the extension
		$ext = strtolower(substr(strrchr($real_filename, '.'), 1));

		// Find out the extension
		if ($ext == 'mp3')
			$mime = 'audio/mpeg';
		elseif ($ext == 'wav')
			$mime = 'audio/x-wav';
		elseif ($ext == 'wma')
			$mime = 'audio/x-ms-wma';
		elseif ($ext == 'mpg' || $ext == 'mpeg')
			$mime = 'video/mpeg';
		elseif ($ext == 'mov')
			$mime = 'video/quicktime';
		elseif ($ext == 'avi')
			$mime = 'video/x-msvideo';
		elseif ($ext == 'wmv')
			$mime = 'video/x-ms-wmv';

		// These are extra posible mime types for custom sections
		// Archives
		elseif ($ext == 'zip')
			$mime = 'application/zip';
		elseif ($ext == 'gz')
			$mime = 'application/x-gzip';
		elseif ($ext == 'tar')
			$mime = 'application/x-tar';
		elseif ($ext == 'rar')
			$mime = 'application/x-rar-compressed';

		// PDF
		elseif ($ext == 'pdf')
			$mime = 'application/pdf';

		// Doc, txt, css
		elseif ($ext == 'doc')
			$mime = 'application/msword';
		elseif ($ext == 'txt')
			$mime = 'text/plain';

		// Else default is...
		else
			$mime = 'application/octet-stream';

		// Set the actual header
		header('Content-Type: ' . $mime);
	}

	// Firefox needs to know the length or else the song only plays a few secs.
	header('Content-Length: ' . filesize($filename));

	// Try to buy some time...
	@set_time_limit(0);

	if (@readfile($filename) == null)
		echo file_get_contents($filename);

	obExit(false);
}

function MediaPopup()
{
	global $txt, $context, $scripturl;

	// Check if they should be here and what to load.
	if (isset($_REQUEST['attach']) && !is_array($_REQUEST['attach']))
	{
		if (isset($_REQUEST['music']) && allowedTo(array('media_center_listen')))
		{
			$context['media_center_popup'] = 'music';
			$context['media_center_file'] = $scripturl . '?action=mediacenterfile;attach=' . $_REQUEST['attach'] . ';music';
		}
		elseif (isset($_REQUEST['video']) && allowedTo(array('media_center_video')))
		{
			$context['media_center_popup'] = 'video';
			$context['media_center_file'] = $scripturl . '?action=mediacenterfile;attach=' . $_REQUEST['attach'] . ';video';
		}
		else
			$context['media_center_popup'] = 'exit';
	}
	else
		redirectexit('action=mediacenter');

	// Load language and template files.
	loadLanguage('MediaCenter');
	loadTemplate('MediaCenter');

	$context['page_title'] = $txt['media_title'];
	$context['template_layers'] = array();
	$context['sub_template'] = 'media_popup';
}
?>